
# q-expansion of newform 114.6.a.d, downloaded from the LMFDB on 13 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 114
weight = 6
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[4], [9], [-91], [-33], [-91], [-610], [-1833], [-361], [-3436], [3562], [322], [7216], [-13664], [-3701], [9203], [29186], [-27804], [43127], [-19428], [7040], [37341], [-4972], [-71196], [-3654], [62362], [-171190], [88590], [117758], [82416], [80414], [-138942], [-318813], [-363929], [-309105], [-436653], [466100], [218686], [-279304], [457854], [-733002], [-247518], [-189158], [330733], [674472], [-942346], [-429505], [-569088], [1001324], [169582], [-405367], [506649], [1347657], [-840812], [1082891], [522416], [-1088949], [-924702], [-1192700], [-1906907], [19066], [-667833], [-1432260], [3089112], [-2782481], [-1093834], [1555776], [35240], [-1648250], [-3746309], [1157233], [-648018], [3319689], [3305920], [1957856], [-1121790], [1033054], [4668765], [-3003103], [3946480], [4380034], [872676], [-1958542], [90666], [3508250], [-4919698], [7926871], [1482796], [1728251], [-552109], [-5549291], [2056327], [-2042792], [-6585644], [-3967144], [-2696111], [8317560], [10019706], [-235936], [914870], [-4041925], [8182930], [-12235697], [10783044], [11076010], [-5855704], [-7265413], [11972139], [-2077886], [10521684], [3582940], [-4386250], [3859585], [11725587], [6818692], [-9819805], [-19601922], [12325219], [968621], [-517653], [7305482], [-21207644], [-5201428], [-8903378], [-14051786], [-5389203], [23089706], [4253550], [-818051], [-24564371], [368658], [-26403673], [27705092], [-27210160], [-14096513], [3588687], [9727171], [-4335984], [-30745136], [-24974448], [-23371575], [-23359098], [-22640325], [26261519], [-15497896], [-6517500], [21051494], [29268426], [-14082492], [-10881665], [7837524], [-3131702], [-18522261], [5510959], [-45681728], [35441942], [-41855334], [12948872], [-34275586], [38139221], [43488130], [-32377116], [41217236], [34023792], [-40342564], [17040062], [22079604], [-17040032], [20186617], [-43933496], [-40097637], [-6320796], [5330174], [53818296], [-41268226], [52769466], [33769461], [13434620], [-38015829], [51432628], [5845619], [37318249], [62402398], [14092666], [-10825316], [68808729], [-39787437], [3335978], [-17048744], [2363258], [-27303648], [38540662], [7440125], [67689550], [-51027576], [16678156], [4952274], [36482213], [-33037110], [3136730], [-16206741], [-16538592], [44552392], [37968604], [-61327630], [-97103020], [-55139941], [28957193], [-5994960], [47437011], [-15054670], [-31503322], [-922138], [9380799], [121194108], [70760106], [-54674712], [-50074656], [-21488520], [76438210], [86939985], [-43757076], [-23046976], [7640486], [115526861], [-116050732], [82036515], [-136724296], [112552512], [-112802716], [23707017], [33228903], [-130709026], [74667246], [-24840536], [-138099355], [-56080575], [-129594795], [-61144027], [168324884], [-82270608], [-105119318], [-110160383], [95035180], [70685876], [27408258], [52068471], [-53124683], [-43898848], [131396072], [-48292689], [-123739862], [-97848355], [70981292], [-178240804], [-27525379], [171011404], [-15708843], [37280366], [92663532], [89568682], [-22258400], [192454908], [107626959], [-60573594], [175653657], [9298234], [183389430], [86470857], [-94793116], [-1103847], [-61844406], [151025344], [-63038187], [-245458849], [-94687108], [140273856], [-187052235], [146067970], [-13405112], [-175616007], [848160], [179630488], [-196083153], [294363720], [189953623], [-34641723], [-33168069], [-274799670], [-88689509], [132545210], [-180751230], [-36919803], [7050006], [-104782048], [-216822148], [-89665162], [134324972], [-28535539], [72669610], [95091357], [-111078335], [-36989362], [30512335], [-152251822], [-235026123], [-159935482], [308043024], [-80048314], [-300356379], [-78141094], [-150821591], [123080838], [-28663880], [-12250224], [89471160], [-158408118], [62623659], [-110392142], [170115371], [289823203], [-101776424], [-95807307], [229100922], [387702842], [-260417202], [-351729960], [143069038], [315637900], [68571952], [376044551], [126463987], [-65692026], [-343659568], [21783565], [70635651], [202200251], [-133967142], [95301774], [303081095], [-129385376], [-339486956], [-47545401], [35677362], [252688988], [-235967440], [-389662922], [405280530], [-454690530], [-129408418], [194313423], [137776282], [217104284], [305204093], [416459898], [-463922548], [-211730042], [414399224], [443160516], [-311459609], [-162284754], [-421079867], [-325546918], [-137249220], [-468442456], [-271442527], [134367494], [315190098], [-534997122], [578182858], [145440132], [193078170], [-325706869], [116138680], [109611407], [262750337], [-208758610], [-154275088], [-256045873], [-225095115], [-174984528], [95676066], [115651409], [-234568931], [-50162475], [469480338], [667453682], [49156336], [402218938], [-354106979], [41200282], [-419109208], [-615766906], [249702960], [237756769], [-255358356], [-306519538], [-35166020], [-37471762], [-242480618], [682830], [-391578940], [-54912983], [469255230], [-528879842], [649999224], [-198573139], [-544047119], [-303040912], [729297466], [-40574233], [724391170], [504009513], [539766334], [549021689], [-163883222], [-138909702], [87591726], [334361300], [-38795982], [78386098], [-538851708], [-204854470], [-300686929], [469006882], [-152142944], [-762802638], [-188453926], [-356097839], [-904152517], [-678562921]]
